{ config, pkgs, lib, ... }:

{
  imports =
    [
      ./hardware-configuration.nix
      ../../configuration/common
      ../../configuration/router
    ];

  boot.loader.grub.enable = true;
  boot.loader.grub.version = 2;
  boot.loader.grub.device = "/dev/sda";

  networking.hostName = "gatekeeper";

  networking.useDHCP = false;
  # Local Router IPs
  networking.interfaces.lo.ipv6.addresses = [
    { address = "fd00:152:152:101::1"; prefixLength = 64; }
    { address = "fd00:152:152::1"; prefixLength = 128; } # Anycast
  ];
  networking.interfaces.lo.ipv4.addresses = [
    { address = "10.152.101.1"; prefixLength = 24; }
    { address = "10.152.0.1"; prefixLength = 32; } # Anycast
  ];
  # Network
  networking.interfaces.ens3.ipv4.addresses = [ { address = "78.47.183.82"; prefixLength = 32; } ];
  networking.interfaces.ens3.ipv6.addresses = [ { address = "2a01:4f8:c0c:15f1::1"; prefixLength = 64; } ];
  networking.defaultGateway = { address = "172.31.1.1"; interface = "ens3"; };
  networking.defaultGateway6 = { address = "fe80::1"; interface = "ens3"; };
  networking.nameservers = [ "213.133.98.98" "213.133.99.99" "213.133.100.100" ];

  networking.nat = {
    enable = true;
    externalInterface = "ens3";
    internalInterfaces = [ "wg-vpn" ];
    internalIPs = [ "10.20.30.0/24" ];
  };

  networking.wireguard.enable = true;
  networking.wireguard.interfaces = {
    wg-vpn = {
      ips = [ "2a01:4f8:c0c:15f1::8001/113" "10.20.30.1/24" ];
      peers = [
        {
          # krypton
          allowedIPs = [ "2a01:4f8:c0c:15f1::8011/128" "10.20.30.11/32" ];
          publicKey = "NA/vdmoKQ5bpPKKrRPUpI17bNPoviRXnZ61iV4Q1VUQ=";
        }
        {
          # krypton full
          allowedIPs = [ "2a01:4f8:c0c:15f1::8111/128" "10.20.30.111/32" ];
          publicKey = "QGQHWwDE1XIeiReFcacLxin4Dqlz1pBXvttFnzBMJSY=";
        }
        {
          # nexus
          allowedIPs = [ "2a01:4f8:c0c:15f1::8012/128" "10.20.30.12/32" ];
          publicKey = "tEJzPPEJkoTPkhzTWyFDZ+5U146ovHA/4Mv3JButSAQ=";
        }
        {
          # terra
          allowedIPs = [ "2a01:4f8:c0c:15f1::8014/128" "10.20.30.14/32" ];
          publicKey = "peZ94x44sMRNqNxcaN+DI2UMwVFzugZjnnbqbxWcBEs=";
        }
        {
          # palladium
          allowedIPs = [ "2a01:4f8:c0c:15f1::8103/128" "10.20.30.103/32" ];
          publicKey = "kxn69ynVyPJeShsAlVz5Xnd7U74GmCAw181b0+/qj3k=";
        }
        {
          allowedIPs = [ "2a01:4f8:c0c:15f1::8104/128" "10.20.30.104/32" ];
          publicKey = "k1eQINwZPRdIEhND5sKAcHMxEpz/Z+B/2ZCdLhHCG3w=";
        }
      ];
      listenPort = 51020;
      allowedIPsAsRoutes = false;
      privateKeyFile = "/var/src/secrets/wireguard/wg-vpn";
    };
  };

  clerie.nginx-port-forward = {
    enable = true;
    tcpPorts."443" = {
      host = "localhost";
      port = 22;
    };
    tcpPorts."2022" = {
      host = "nonat.net.clerie.de";
      port = 22;
    };
    tcpPorts."25567" = {
      host = "minecraft-1.net.clerie.de";
      port = 25567;
    };
    tcpPorts."25568" = {
      host = "minecraft-1.net.clerie.de";
      port = 25568;
    };
  };

  system.stateVersion = "21.03";
}
